unit euler;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, diagramm, fahrzeug, getriebe, motor, math, controls, graphics;

function f(index: integer; v: Double): Double;
procedure EulerEx();
function M_M(v: Double): Double;
procedure ZeichneKurve(x, y: array of Double; index: Integer);

implementation
const g = 9.81;
var Gang: Integer = 0;

function f(index: integer; v: Double): Double;
var
  iD, m, rdyn, fR, Steig, alpha, cW, A, rho: Double;
  begin
    // Zunächst Einlesen der Fahrzeugparameter
      iD := StrToFloat(F_Get.iD.Text);
    rdyn := StrToFloat(F_Bereif.dynH.Text);
       m := StrToFloat(F_Bereif.Masse.Text);
      fR := StrToFloat(F_Bereif.Roll.Text);
   Steig := StrToFloat(F_Bereif.Steig.Text);
   alpha := arctan(Steig/100);
      cW := StrToFloat(F_Bereif.cW_Wert.Text);
       A := StrToFloat(F_Bereif.Flaeche.Text);
     rho := StrToFloat(F_Bereif.Dichte.Text);
    case index of
      1: f := v;
      2: f := M_M(v)*iG[Gang]*iD/m/km[Gang]/rdyn - g*(fR + sin(alpha)) - cW*A*rho/2/m*v*v;
      3: f := -g*(fR + sin(alpha)) - cW*A*rho/2/m*v*v; // Während der Schaltpause kein Motormoment
    end;
  end;

function M_M(v: Double): Double;
var
  nM, iD, rdyn: Double;
begin
  // Differenzial und rdyn Einlesen
  iD := StrToFloat(F_Get.iD.Text);
  rdyn := StrToFloat(F_Bereif.dynH.Text);
  // Aus der übergebenen Fahrgeschwindigkeit die Drehzahl ermitteln
  nM := v*iG[Gang]*iD/rdyn*30/pi;
  // Aus der Motordrehzahl das Moment über die Kubische Interpolation Berechnen
  M_M := F_Mot.Moment.Calculate(nM);
end;

procedure EulerEx();
var
  dt, iD, rdyn, n0, nOG, vSP, tEG, tEP: Double;
  t, v: array of Double;
  i, j, k, n, AnzGang, TE: Integer;
begin // Vorbereitungen für Euler und Schaltvorgänge mit Schaltpausen

       TE := StrToInt(F_Diag.SimEnde.Text); //Simulationsende
       dt := StrToFloat(F_Diag.Schritt.Text); // Schrittweite der Simulation
        n := round(TE/dt); //Anzahl der Simulationsschritte (Arraygröße)
       iD := StrToFloat(F_Get.iD.Text); //Differenzialübersetzung
     rdyn := StrToFloat(F_Bereif.dynH.Text); //Dynamischer Reifenradius
       n0 := StrToFloat(F_Mot.KennfeldGrid.Cells[0,1]); // Startdrehzahl einlesen
      nOG := StrToFloat(F_Mot.n_Pmax.Text); // Schaltpunktdrehzahl

  AnzGang := StrToInt(F_Get.AnzGang.Text); //Anzahl der Getriebegänge
        i := 0; //Initialisieren des Indexzählers für die Arrays t und v
        j := 0; //Index für das Aneinanderreihen der Arrays t und v
     Gang := 0; //Initialisieren der Getriebeschaltung

  SetLength(t,n);
  SetLength(v,n);
  t[0] := 0.0; // Erstinitialisierung von t zum Zeitpunkt Null
  v[0] := n0 * rdyn / iG[0] / iD * pi / 30; // Beginn im ersten Gang bei Leerlauf- bzw. der ersten Kennfelddrehzahl

  //*****Hauptprogrammschleife******
for k:=0 to AnzGang-1 do //Schleife über die Anzahl der Gänge
 begin
  Gang := k;
  vSP := nOG * rdyn / iG[Gang] / iD *pi / 30;

  for i:=j to (n-1) do //Beschleunigen
    begin
      t[i+1] := (i+1)*dt; // Zeitvektor
      v[i+1] := v[i] + dt * f(2, v[i]); // Geschwindigkeit
      if (v[i+1]>=vSP) then break;
    end; //End "for" Beschleunigen
    j := i;
  tEG := t[i+1];

  if k<=(AnzGang-2) then //Auswahl Schaltpausen
    tEP := tEG + tPs[k]
  else
    tEP := tEG + tPs[AnzGang-2];

  for i:=j to (n-1) do //Schaltpausen
    begin
      t[i+1] := (i+1)*dt;
      v[i+1] := v[i] + dt * f(3, v[i]);
      if (t[i+1]>=tEP) then break;
    end; //End "for" Schaltpausen

  j := i; //Arrray neu ansetzen

end; //Ende "for" *****Hauptprogrammschleife*****
ZeichneKurve(t,v,j);

end; //Ende EulerEx();

procedure ZeichneKurve(x, y: array of Double; index: Integer);
var i: Integer;
  begin
    for i:=0 to index do
      F_Diag.v_t1Line.AddXY(x[i],y[i]*3.6);
    F_Diag.v_t1Line.Add(NaN);
  end;

end.

